package org.bidib.wizard.comm;

import java.util.BitSet;
import java.util.Calendar;
import java.util.Collection;
import java.util.Set;

import org.bidib.jbidibc.Feature;
import org.bidib.jbidibc.FirmwareUpdateStat;
import org.bidib.jbidibc.Node;
import org.bidib.jbidibc.ProtocolVersion;
import org.bidib.jbidibc.SoftwareVersion;
import org.bidib.jbidibc.enumeration.FirmwareUpdateOperation;
import org.bidib.jbidibc.enumeration.IdentifyState;
import org.bidib.wizard.comm.listener.CommunicationListener;
import org.bidib.wizard.mvc.main.model.Accessory;
import org.bidib.wizard.mvc.main.model.AnalogPort;
import org.bidib.wizard.mvc.main.model.FeedbackPort;
import org.bidib.wizard.mvc.main.model.Flag;
import org.bidib.wizard.mvc.main.model.InputPort;
import org.bidib.wizard.mvc.main.model.LightPort;
import org.bidib.wizard.mvc.main.model.Macro;
import org.bidib.wizard.mvc.main.model.MotorPort;
import org.bidib.wizard.mvc.main.model.ServoPort;
import org.bidib.wizard.mvc.main.model.SoundPort;
import org.bidib.wizard.mvc.main.model.SwitchPort;

public interface Communication {

    /**
     * Open the comm port.
     * @param commPort
     *            the connected comm port
     */
    void open(String commPort);

    void addCommunicationListener(CommunicationListener listener);

    /**
     * Returns the list of features of the specified node
     * 
     * @param node
     *            the node
     * @return list of features
     */
    Set<Feature> getFeatures(Node node);

    /**
     * Read a single feature of the specified node.
     * 
     * @param node
     *            the node
     * @param featureNumber
     *            the feature number
     * @return the feature
     */
    Feature readFeature(Node node, int featureNumber);

    /**
     * Write a single feature to the specified node.
     * 
     * @param node
     *            the node
     * @param feature
     *            the feature
     */
    void writeFeature(Node node, Feature feature);

    /**
     * Clear all cached data of a node
     * 
     * @param node
     *            the node
     */
    void clearCachedData(Node node);

    void activateAnalogPort(Node node, int port, AnalogPortStatus status);

    void activateLightPort(Node node, int port, LightPortStatus status);

    void activateMotorPort(Node node, int port, MotorPortStatus status);

    void activateServoPort(Node node, int port, int value);

    void activateSoundPort(Node node, int port, SoundPortStatus status);

    void activateSwitchPort(Node node, int port, SwitchPortStatus status);

    void boosterOff(Node node);

    void boosterOn(Node node);

    void boosterQuery(Node node);

    void disable(Node node);

    /**
     * Release the node to allow the node to send spontaneous messages.
     * 
     * @param node
     *            the node
     */
    void enable(Node node);

    Collection<Accessory> getAccessories(Node node);

    int getAccessoryLength(Node node);

    /**
     * Get the analog ports of the node. If the analog ports are not cached, a request to get the number of analog ports from
     * the bidib node is performed.
     * 
     * @param node
     *            the node
     * @return the collection of analog ports
     */
    Collection<AnalogPort> getAnalogPorts(Node node);

    /**
     * Get the maximum booster current of the node.
     * 
     * @param node
     *            the node
     * @return the maximum booster current value
     */
    int getBoosterMaximumCurrent(Node node);

    /**
     * Get the feedback ports. If the feedback ports are not cached, a request to get the number of feedback ports from
     * the bidib node is performed.
     * 
     * @param node
     *            the node
     * @return the collection of feedback ports
     */
    Collection<FeedbackPort> getFeedbackPorts(Node node);

    /**
     * Get the feedback state, address state and confidence of the feedback ports
     * 
     * @param node
     *            the node to query
     * @param begin
     *            the start range of port number
     * @param end
     *            the end range of port number
     */
    void getFeedbackPortStatus(Node node, int begin, int end);

    Collection<Flag> getFlags();

    /**
     * Get the input ports of the node. If the input ports are not cached, a request to get the number of input ports from
     * the bidib node is performed.
     * 
     * @param node
     *            the node
     * @return the collection of input ports
     */
    Collection<InputPort> getInputPorts(Node node);

    void getInputPortStatus(Node node, int port);

    Collection<LightPort> getLightPorts(Node node);

    /**
     * Returns the supported number of entries per macro. This value is dependent on the device that runs on the node.
     * 
     * @param node
     *            the node
     * @return the number of entries per macro
     */
    int getMacroLength(Node node);

    /**
     * Get the content of the macro.
     * 
     * @param node
     *            the node
     * @param macro
     *            the current macro instance
     * @return the provided macro instance with the content retrieved from the device
     */
    Macro getMacroContent(Node node, Macro macro);

    Collection<Macro> getMacros(Node node);

    /**
     * Get the motor ports of the node. If the motor ports are not cached, a request to get the number of motor ports from
     * the bidib node is performed.
     * 
     * @param node
     *            the node
     * @return the collection of motor ports
     */
    Collection<MotorPort> getMotorPorts(Node node);

    Collection<Node> getNodes();

    ProtocolVersion getProtocolVersion(Node node);

    Collection<ServoPort> getServoPorts(Node node);

    Collection<SoundPort> getSoundPorts(Node node);

    int getStorableMacroCount(Node node);

    Collection<SwitchPort> getSwitchPorts(Node node);

    /**
     * Get the software version of the node
     * 
     * @param node
     *            the node
     * @return the current software version
     */
    SoftwareVersion getSoftwareVersion(Node node);

    void identify(Node node, IdentifyState state);

    Boolean isAddressMessagesEnabled(Node node);

    Boolean isCommandStation(Node node);

    Boolean isDccStartEnabled(Node node);

    Boolean isExternalStartEnabled(Node node);

    Boolean isFeedbackMessagesEnabled(Node node);

    Boolean isKeyMessagesEnabled(Node node);

    /**
     * Check if the node supports FW update
     * 
     * @param node
     *            the node
     * @return the node supports FW update
     */
    boolean isUpdatable(Node node);

    int readCv(Node node, int cvNumber);

    void reloadAccessory(Node node, Accessory accessory);

    /**
     * Load the macro with the specified macro id from the node.
     * 
     * @param node
     *            the node
     * @param macro
     *            the macro
     */
    void reloadMacro(Node node, Macro macro);

    /**
     * Reset the node.
     * 
     * @param node
     *            the node, if null the root node is reset
     */
    void reset(Node node);

    void saveAccessory(Node node, Accessory accessory);

    /**
     * Transfer the macro to the node and store in permanent memory.
     * 
     * @param node
     *            the node
     * @param macro
     *            the macro
     */
    void saveMacro(Node node, Macro macro);

    FirmwareUpdateStat sendFirmwareUpdateOperation(Node node, FirmwareUpdateOperation operation, byte... data);

    /**
     * Send the current model time to the system.
     * 
     * @param time
     *            the current model time
     * @param factor
     *            the acceleration factor
     */
    void sendTime(Calendar time, int factor);

    void setAddressMessagesEnabled(Node node, boolean enabled);

    void setBinaryState(Node node, int address, int state, boolean value);

    void setDccStartEnabled(Node node, boolean enabled);

    void setExternalStartEnabled(Node node, boolean enabled);

    void setFeedbackMessagesEnabled(Node node, boolean enabled);

    void setKeyMessagesEnabled(Node node, boolean enabled);

    void setLightPortParameters(Node node, int port, int pwmMin, int pwmMax, int dimMin, int dimMax);

    void setServoPortParameters(Node node, int port, int trimDown, int trimUp, int speed);

    void setSpeed(
        Node node, int address, SpeedSteps speedSteps, Integer speed, Direction direction, BitSet activeFunctions,
        BitSet functions);

    void startAccessory(Node node, Accessory accessory, int ascpect);

    void startMacro(Node node, Macro macro);

    void stopMacro(Node node, Macro macro);

    /**
     * Transfer the macro to the node but don't store in permanent memory.
     * 
     * @param node
     *            the node
     * @param macro
     *            the macro
     */
    void transferMacro(Node node, Macro macro);

    void writeCv(Node node, int cvNumber, int cvValue);
}
