package org.bidib.wizard.comm;

import java.io.IOException;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.concurrent.Semaphore;

import org.bidib.jbidibc.Bidib;
import org.bidib.jbidibc.MessageReceiver;
import org.bidib.wizard.comm.bidib.BidibCommunication;
import org.bidib.wizard.comm.listener.CommunicationListener;
import org.bidib.wizard.comm.mock.MockCommunication;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.preferences.model.CommPort;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public final class CommunicationFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(CommunicationFactory.class.getName());

    private static Communication instance = null;

    private static Semaphore semaphore = new Semaphore(1);

    private static Set<CommunicationListener> listeners = new LinkedHashSet<CommunicationListener>();

    static {
        Preferences.getInstance().addPreferencesListener(new PreferencesListener() {
            @Override
            public void commPortChanged(String commPort) {
                LOGGER.debug("The comm port has changed: {}", commPort);
                // check if the comm port has changed
                synchronized (CommunicationFactory.class) {
                    try {
                        semaphore.acquire();
                        if (instance != null) {
                            if (instance instanceof BidibCommunication) {
                                // the real world communication
                                BidibCommunication communication = (BidibCommunication) instance;
                                if (!commPort.equals(communication.getConnectedCommPort())) {
                                    // the comm port has changed
                                    LOGGER.debug("Disable the message receiver.");
                                    MessageReceiver.disable();
                                    LOGGER.debug("Close the Bidib port.");
                                    try {
                                        Bidib.getInstance().close();
                                    }
                                    catch (IOException ex) {
                                        LOGGER.warn("Close Bidib instance failed.", ex);
                                    }
                                    instance = null;
                                    LOGGER.info("Released instance of BidibCommunication.");
                                }
                            }
                        }
                    }
                    catch (InterruptedException e) {
                        LOGGER.warn("Acquire semaphore failed.", e);
                        throw new RuntimeException(e);
                    }
                    finally {
                        semaphore.release();
                    }
                }
            }

            @Override
            public void mockChanged(boolean mock) {
                try {
                    semaphore.acquire();
                    LOGGER.debug("The mock has changed, mock: {}", mock);
                    if (mock) {
                        LOGGER.debug("Disable the message receiver.");
                        MessageReceiver.disable();
                        LOGGER.debug("Close the Bidib port.");
                        Bidib.getInstance().close();
                    }
                    instance = null;
                }
                catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                catch (IOException e) {
                    LOGGER.warn("Close the bidib port failed.", e);
                    throw new RuntimeException(e);
                }
                finally {
                    semaphore.release();
                }
            }

            @Override
            public void startTimeChanged(Date startTime) {
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
            }
        });
    }

    private CommunicationFactory() {
    }

    public synchronized static Communication getInstance() {
        if (instance == null) {
            try {
                semaphore.acquire();

                fireStatus(Resources.getString(CommunicationFactory.class, "open-port"), -1);
                if (Preferences.getInstance().isMock()) {
                    instance = new MockCommunication();
                    instance.addCommunicationListener(new CommunicationListener() {
                        @Override
                        public void opened(String port) {
                            fireOpened(port);
                        }

                        @Override
                        public void status(String statusText, int displayDuration) {
                            fireStatus(statusText, displayDuration);
                        }
                    });
                    instance.open("mock");
                }
                else {
                    String commPort = "unknown";
                    try {
                        commPort = Preferences.getInstance().getCommPort();
                        LOGGER.debug("Start to open the Bidib port: {}", commPort);
                        if (!CommPort.NONE.equals(commPort)) {

                            instance = new BidibCommunication();
                            instance.addCommunicationListener(new CommunicationListener() {
                                @Override
                                public void opened(String port) {
                                    fireOpened(port);
                                }

                                @Override
                                public void status(String statusText, int displayDuration) {
                                    fireStatus(statusText, displayDuration);
                                }
                            });
                            instance.open(commPort);
                        }
                        else {
                            LOGGER.info("No comm port selected, do not create BidibCommunication.");
                            fireStatus(Resources.getString(CommunicationFactory.class, "no-port-selected"), -1);
                        }
                    }
                    catch (Exception e) {
                        LOGGER.warn("Open Bidib failed.", e);

                        fireStatus(Resources.getString(CommunicationFactory.class, "open-port-failed") + commPort, -1);
                    }
                }
            }
            catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
            finally {
                semaphore.release();
            }
        }
        return instance;
    }

    public static void addCommunicationListener(CommunicationListener listener) {
        LOGGER.debug("Adding communication listener: {}", listener);
        listeners.add(listener);
    }

    private static void fireOpened(String port) {
        for (CommunicationListener listener : listeners) {
            listener.opened(port);
        }
    }

    private static void fireStatus(String statusText, int displayDuration) {
        for (CommunicationListener listener : listeners) {
            listener.status(statusText, displayDuration);
        }
    }
}
