package org.bidib.wizard.mvc.features.view.panel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.bidib.jbidibc.Feature;
import org.bidib.jbidibc.enumeration.FeatureEnum;
import org.bidib.wizard.mvc.features.model.FeaturesModel;
import org.bidib.wizard.mvc.features.model.listener.FeatureListener;
import org.bidib.wizard.mvc.features.view.panel.listener.FeaturesWriteListener;
import org.bidib.wizard.utils.IntegerEditor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FeaturesPanel extends JPanel {
    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = LoggerFactory.getLogger(FeaturesPanel.class);

    private final Set<FeaturesWriteListener> listeners = new LinkedHashSet<FeaturesWriteListener>();

    private FeatureTableModel featureTableModel;

    private JTable table;

    private final FeaturesModel model;

    int featuresAdded = 0;

    private TableRowSorter<TableModel> sorter;

    public FeaturesPanel(final FeaturesModel model) {
        this.model = model;
        featureTableModel = new FeatureTableModel();
        table = new JTable(
            featureTableModel) {
            private static final long serialVersionUID = 1L;

            @Override
            public TableCellEditor getCellEditor(int row, int column) {
                if (column == 1) {
                    // must use the sorter model to get the correct row
                    row = sorter.convertRowIndexToModel(row);
                    Feature val = ((FeatureTableModel) getModel()).getRowAt(row);
                    if (val != null) {
                        FeatureEnum feature = val.getFeatureEnum();
                        if (feature != null) {
                            Integer min = 0;
                            if (feature.getMin() != null) {
                                min = feature.getMin();
                            }
                            Integer max = 0;
                            if (feature.getMax() != null) {
                                max = feature.getMax();
                            }
                            LOGGER.debug("Limited editor, min: {}, max: {}", min, max);
                            IntegerEditor editor = new IntegerEditor(min, max);
                            return editor;
                        }
                    }
                }
                return super.getCellEditor(row, column);
            }
        };
        sorter = new TableRowSorter<TableModel>(table.getModel());
        List<RowSorter.SortKey> sortKeys = new ArrayList<RowSorter.SortKey>();
        sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
        sorter.setSortKeys(sortKeys);
        table.setRowSorter(sorter);
        table.setDefaultEditor(Integer.class, new IntegerEditor(0, 255));
        add(table);

        model.addFeatureListener(new FeatureListener() {

            @Override
            public void featureChanged(Feature feature) {
                LOGGER.debug("The following feature has changed: {}", feature);
                showFeature(feature);
            }
        });

        showFeatures(model.getFeatures());
    }

    public void addFeaturesWriteListener(FeaturesWriteListener listener) {
        listeners.add(listener);
    }

    public void showFeatures(Collection<Feature> features) {
        LOGGER.debug("Show features: {}", features);
        featureTableModel.clear();

        for (Feature feature : features) {
            LOGGER.debug("Add feature: {}", feature);
            showFeature(feature);
        }
    }

    public JTable getTable() {
        return table;
    }

    private void showFeature(Feature feature) {
        LOGGER.debug("Show feature: {}", feature);

        featureTableModel.updateRow(feature);
    }

    /**
     * Write the features to the model and notify the listener of the 
     * changed features. If no exception is thrown the model is updated 
     * with the new feature values.
     */
    public void writeFeatures() {
        if (table.getCellEditor() != null) {
            table.getCellEditor().stopCellEditing();
        }

        List<Feature> features = featureTableModel.getFeatures();

        // check the changed data
        Collection<Feature> featuresToWrite = model.prepareUpdateFeatures(features);

        // write the changed features to the node
        fireWrite(featuresToWrite);

        // update the changed features in the model
        model.setFeatures(featuresToWrite);
    }

    private void fireWrite(Collection<Feature> features) {
        for (FeaturesWriteListener l : listeners) {
            l.write(features);
        }
    }

}
