package org.bidib.wizard.mvc.main.view.component;

import static java.util.concurrent.TimeUnit.MILLISECONDS;
import static java.util.concurrent.TimeUnit.SECONDS;

import java.awt.Font;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;

import javax.swing.JLabel;

import org.bidib.wizard.comm.Communication;
import org.bidib.wizard.comm.CommunicationFactory;
import org.bidib.wizard.comm.listener.CommunicationListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DigitalClock extends JLabel implements CommunicationListener {
    private static final Logger LOGGER = LoggerFactory.getLogger(DigitalClock.class);

    private static final long serialVersionUID = -1929458366104742932L;

    private static final long INITIAL_DELAY = 60000;

    private static final long PERIOD = 60000;

    private Communication communication;

    private final SimpleDateFormat format = new SimpleDateFormat("HH:mm:ss");

    private final Calendar startTime = Calendar.getInstance();

    private long timeOffset = 0;

    private int timeFactor = 0;

    private boolean stopped = false;

    private final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(1);

    private ScheduledFuture<?> modelClockTimer = null;

    // helper object to lock the communication access
    private Object commLock = new Object();

    public DigitalClock(Date startTime, int timeFactor) {
        setFont(new Font("SansSerif", Font.BOLD, 20));
        setStartTime(startTime);
        setTimeFactor(timeFactor);

        CommunicationFactory.addCommunicationListener(this);
    }

    private Calendar getStartTime() {
        return startTime;
    }

    private Calendar getTime() {
        Calendar result = (Calendar) startTime.clone();
        long diff = System.currentTimeMillis() - result.getTime().getTime() - timeOffset;

        result.add(Calendar.MILLISECOND, (int) diff * timeFactor);
        return result;
    }

    private int getTimeFactor() {
        return timeFactor;
    }

    public void setStartTime(Date startTime) {
        Calendar start = Calendar.getInstance();

        start.setTime(startTime);
        this.startTime.set(Calendar.HOUR_OF_DAY, start.get(Calendar.HOUR_OF_DAY));
        this.startTime.set(Calendar.MINUTE, start.get(Calendar.MINUTE));
        this.startTime.set(Calendar.SECOND, start.get(Calendar.SECOND));
        timeOffset = System.currentTimeMillis() - this.startTime.getTime().getTime();
    }

    public void setTimeFactor(int timeFactor) {
        this.timeFactor = timeFactor;
        restartModelClock();
    }

    public void start() {
        stopped = false;
        synchronized (commLock) {
            if (communication != null) {
                communication.sendTime(getStartTime(), getTimeFactor());
            }
        }
    }

    private void restartModelClock() {
        if (modelClockTimer != null) {
            modelClockTimer.cancel(true);
        }

        synchronized (commLock) {
            if (communication == null) {
                return;
            }
        }
        timeOffset = System.currentTimeMillis() - this.startTime.getTime().getTime();
        modelClockTimer = scheduler.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                if (!stopped) {
                    Calendar currentTime = getTime();
                    LOGGER.info("Send current time: {}", currentTime.getTime());
                    synchronized (commLock) {
                        communication.sendTime(currentTime, getTimeFactor());
                    }
                }
            }
        }, INITIAL_DELAY / timeFactor, PERIOD / timeFactor, MILLISECONDS);
    }

    public void stop() {
        LOGGER.info("Stop the digital clock.");
        stopped = true;
        synchronized (commLock) {
            if (communication != null) {
                communication.sendTime(getStartTime(), 0);
            }
        }
    }

    @Override
    public void opened(String port) {
        // communication is opened, start the timers
        synchronized (commLock) {
            communication = CommunicationFactory.getInstance();
        }
        LOGGER.debug("Using communication: {}", communication);
        scheduler.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                if (!stopped) {
                    setText(format.format(getTime().getTime()));
                }
            }
        }, 0, 1, SECONDS);

        restartModelClock();
    }

    @Override
    public void status(String statusText, int displayDuration) {

    }
}
