package org.bidib.wizard.mvc.main.view.panel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;

import org.bidib.wizard.comm.BoosterStatus;
import org.bidib.wizard.locale.Resources;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.listener.BoosterStatusListener;
import org.bidib.wizard.mvc.main.view.panel.listener.StatusListener;
import org.bidib.wizard.mvc.preferences.model.Preferences;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.hansolo.steelseries.gauges.Radial1Vertical;
import eu.hansolo.steelseries.tools.Section;

public class BoosterPanel extends JPanel {
    private static final Logger LOGGER = LoggerFactory.getLogger(BoosterPanel.class);

    private static final long serialVersionUID = -6498187029377078624L;

    private Collection<StatusListener> statusListeners = new LinkedList<StatusListener>();

    private final MainModel model;

    public BoosterPanel(final MainModel model) {
        this.model = model;
        setLayout(new GridBagLayout());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 1;

        JPanel statusPanel = new JPanel(new GridBagLayout());

        statusPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources.getString(
            getClass(), "status")
            + ":"));

        final JToggleButton statusButton = addStatusButton(statusPanel);
        final JTextField statusText = addStatusText(statusPanel);

        add(statusPanel, c);

        JPanel diagnosticPanel = new JPanel(new GridBagLayout());

        diagnosticPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Resources
            .getString(getClass(), "diagnostic")
            + ":"));

        final Radial1Vertical voltMeter = addVoltMeter(diagnosticPanel);
        final Radial1Vertical thermoMeter = addThermoMeter(diagnosticPanel);
        final Radial1Vertical ampereMeter = addAmpereMeter(diagnosticPanel);

        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.gridy++;
        c.weighty = 1;
        add(diagnosticPanel, c);

        model.addBoosterStatusListener(new BoosterStatusListener() {
            @Override
            public void currentChanged(final int current) {
                if (SwingUtilities.isEventDispatchThread()) {
                    internalCurrentChanged(current);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalCurrentChanged(current);
                        }
                    });
                }
            }

            private void internalCurrentChanged(final int current) {
                LOGGER.debug("Current has changed: {}", current);
                ampereMeter.setValueAnimated(current);
            }

            @Override
            public void maximumCurrentChanged(final int maximumCurrent) {

                if (SwingUtilities.isEventDispatchThread()) {
                    internalMaximumCurrentChanged(maximumCurrent);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalMaximumCurrentChanged(maximumCurrent);
                        }
                    });
                }
            }

            private void internalMaximumCurrentChanged(int maximumCurrent) {
                LOGGER.debug("Maximum current has changed: {}", maximumCurrent);

                if (maximumCurrent > 0) {
                    ampereMeter.setMaxValue(maximumCurrent);

                    Section[] sections = new Section[3];

                    sections[0] = new Section(0, maximumCurrent / 2, Color.GREEN);
                    sections[1] = new Section(maximumCurrent / 2, 3 * maximumCurrent / 4, Color.ORANGE);
                    sections[2] = new Section(3 * maximumCurrent / 4, maximumCurrent, Color.RED);
                    ampereMeter.setSections(sections);
                    ampereMeter.setValue(model.getBoosterCurrent());
                }
            }

            @Override
            public void stateChanged(final BoosterStatus status) {
                if (SwingUtilities.isEventDispatchThread()) {
                    internalStateChanged(status);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalStateChanged(status);
                        }
                    });
                }
            }

            private void internalStateChanged(BoosterStatus status) {
                LOGGER.debug("Status of booster has changed: {}", status);
                setStatusButtonState(statusButton, status);
                setStatusText(statusText, status);
            }

            @Override
            public void temperatureChanged(final int temperature) {
                if (SwingUtilities.isEventDispatchThread()) {
                    internalTemperatureChanged(temperature);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalTemperatureChanged(temperature);
                        }
                    });
                }
            }

            private void internalTemperatureChanged(int temperature) {
                thermoMeter.setValueAnimated(temperature);
            }

            @Override
            public void voltageChanged(final int voltage) {
                if (SwingUtilities.isEventDispatchThread()) {
                    internalVoltageChanged(voltage);
                }
                else {
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            internalVoltageChanged(voltage);
                        }
                    });
                }
            }

            private void internalVoltageChanged(int voltage) {
                voltMeter.setValueAnimated(voltage / 10d);
            }
        });

        Preferences.getInstance().addPreferencesListener(new PreferencesListener() {
            @Override
            public void commPortChanged(String commPort) {
            }

            @Override
            public void mockChanged(boolean mock) {
            }

            @Override
            public void startTimeChanged(Date startTime) {
            }

            @Override
            public void timeFactorChanged(int timeFactor) {
            }
        });

    }

    private Radial1Vertical addAmpereMeter(JPanel panel) {
        Radial1Vertical result = new Radial1Vertical();

        result.setFrameVisible(true);
        result.setLcdDecimals(0);
        result.setLcdUnitStringVisible(false);
        result.setLedVisible(false);
        result.setSectionsVisible(true);
        result.setTitle(Resources.getString(getClass(), "current"));
        result.setTrackVisible(false);
        result.setUnitString("mA");

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.BOTH;
        c.gridheight = 2;
        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.75;
        c.weighty = 1;
        panel.add(result, c);
        return result;
    }

    private JToggleButton addStatusButton(JPanel panel) {
        final JToggleButton result = new JToggleButton();

        result.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JToggleButton button = (JToggleButton) e.getSource();

                if (button.isSelected()) {
                    fireSwitchedOn();
                }
                else {
                    fireSwitchedOff();
                }
            }
        });
        setStatusButtonState(result, BoosterStatus.OFF);

        Dimension size = result.getPreferredSize();

        result.setPreferredSize(new Dimension(60, size.height));

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.CENTER;
        c.fill = GridBagConstraints.NONE;
        c.gridx = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.2;
        panel.add(result, c);
        return result;
    }

    private JTextField addStatusText(JPanel panel) {
        JTextField result = new JTextField();

        result.setEditable(false);
        setStatusText(result, model.getBoosterStatus());

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.LINE_START;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.8;
        panel.add(result, c);
        return result;
    }

    public void addStatusListener(StatusListener l) {
        statusListeners.add(l);
    }

    private Radial1Vertical addThermoMeter(JPanel panel) {
        Radial1Vertical result = new Radial1Vertical();

        result.setFrameVisible(true);
        result.setLcdDecimals(0);
        result.setLcdUnitStringVisible(false);
        result.setLedVisible(false);
        result.setMaxValue(100);
        result.setSectionsVisible(true);
        result.setTitle(Resources.getString(getClass(), "temperature"));
        result.setTrackVisible(false);
        result.setUnitString("°C");

        Section[] sections = new Section[3];

        sections[0] = new Section(0, 60, Color.GREEN);
        sections[1] = new Section(60, 85, Color.ORANGE);
        sections[2] = new Section(85, result.getMaxValue(), Color.RED);
        result.setSections(sections);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 1;
        c.gridy = 0;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.25;
        c.weighty = 1;
        panel.add(result, c);
        return result;
    }

    private Radial1Vertical addVoltMeter(JPanel panel) {
        Radial1Vertical result = new Radial1Vertical();

        result.setFrameVisible(true);
        result.setLcdDecimals(0);
        result.setLcdUnitStringVisible(false);
        result.setLedVisible(false);
        result.setMaxValue(30);
        result.setSectionsVisible(true);
        result.setTitle(Resources.getString(getClass(), "voltage"));
        result.setTrackVisible(false);
        result.setUnitString("V");

        Section[] sections = new Section[5];

        sections[0] = new Section(0, 10, Color.RED);
        sections[1] = new Section(10, 12, Color.ORANGE);
        sections[2] = new Section(12, 17, Color.GREEN);
        sections[3] = new Section(17, 20, Color.ORANGE);
        sections[4] = new Section(20, result.getMaxValue(), Color.RED);
        result.setSections(sections);

        GridBagConstraints c = new GridBagConstraints();

        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.fill = GridBagConstraints.BOTH;
        c.gridx = 1;
        c.gridy = 1;
        c.insets = new Insets(5, 5, 5, 5);
        c.weightx = 0.25;
        c.weighty = 1;
        panel.add(result, c);
        return result;
    }

    private void fireSwitchedOff() {
        for (StatusListener l : statusListeners) {
            l.switchedOff();
        }
    }

    private void fireSwitchedOn() {
        for (StatusListener l : statusListeners) {
            l.switchedOn();
        }
    }

    public String getName() {
        return Resources.getString(getClass(), "name");
    }

    private void setStatusButtonState(JToggleButton statusButton, BoosterStatus status) {
        if (status != null) {
            if ((status.getBoosterState().getType() & 0x80) == 0) {
                statusButton.setSelected(false);
                statusButton.setText(Resources.getString(getClass(), "on"));
            }
            else {
                statusButton.setSelected(true);
                statusButton.setText(Resources.getString(getClass(), "off"));
            }
        }
    }

    private void setStatusText(JTextField statusText, BoosterStatus status) {
        statusText.setText(status != null ? status.toString() : "");
    }
}
