package org.bidib.wizard.mvc.preferences.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.Properties;

import org.bidib.wizard.main.BiDiBWizard;
import org.bidib.wizard.mvc.preferences.model.listener.PreferencesListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.joran.JoranConfigurator;
import ch.qos.logback.core.joran.spi.JoranException;
import ch.qos.logback.core.util.StatusPrinter;

/**
 * The preferences of the Bidib-Wizard application.
 *
 */
public final class Preferences {
    private static final Logger LOGGER = LoggerFactory.getLogger(Preferences.class);

    // the default version of preferences
    private static final String DEFAULT_VERSION = "1.4";

    private static final String PROPERTY_COMM_PORT = "commPort";

    private static final String PROPERTY_LABEL_PATH = "labelPath";

    private static final String PROPERTY_LOG_FILE_PATH = "logFilePath";

    @Deprecated
    private static final String PROPERTY_LOG_FILE = "logFile";

    @Deprecated
    private static final String PROPERTY_RXTX_LOG_FILE = "rxtxLogFile";

    private static final String PROPERTY_VERSION = "version";

    private static final String PROPERTY_MOCK = "mock";

    private static final String PROPERTY_START_TIME = "startTime";

    private static final String PROPERTY_TIME_FACTOR = "timeFactor";

    private static final String PROPERTY_WORKING_DIRECTORY = "workingDirectory";

    private static Preferences instance = null;

    private static String preferencesPath = null;

    private final Collection<PreferencesListener> listeners = new LinkedList<PreferencesListener>();

    private final SimpleDateFormat format = new SimpleDateFormat("HH:mm");

    private Properties properties;

    private String preferencesVersion;

    private boolean reloadLogConfigurationPending;

    private Preferences() {
        properties = new Properties();
    }

    private void load() {
        FileInputStream fis = null;

        try {
            fis = new FileInputStream(new File(preferencesPath));
            properties.load(fis);
        }
        catch (FileNotFoundException e) {
            // create initial properties
            throw new IllegalArgumentException("No preferences found at: " + preferencesPath, e);
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
        finally {
            if (fis != null) {
                try {
                    fis.close();
                }
                catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    public static synchronized Preferences getInstance() {
        if (instance == null) {
            Preferences tempInstance = new Preferences();
            try {
                // try to load the properties from the file
                tempInstance.load();
                tempInstance.checkVersionUpgrade();

                // no problem detected
                instance = tempInstance;
            }
            catch (IllegalArgumentException ex) {
                // the file is not available, create new file with default values
                tempInstance.loadDefaultValues();
                tempInstance.save(tempInstance.createPreferencesModel());
                instance = tempInstance;
            }
        }
        return instance;
    }

    private void checkVersionUpgrade() {
        boolean changes = false;
        boolean mustUpgrade = false;
        // check the version and update if necessary
        String[] version = getVersion();
        String[] currentVersion = DEFAULT_VERSION.split("\\.");
        if (!currentVersion[0].equals(version[0])) {
            LOGGER.info("The major version has changed!");
            mustUpgrade = true;
        }
        else if (!currentVersion[1].equals(version[1])) {
            LOGGER.info("The minor version has changed!");
            mustUpgrade = true;
        }

        if (mustUpgrade) {
            // check all properties
            String logFilePath = getLogFilePath();
            File dir = new File(logFilePath);
            if (dir.exists()) {
                LOGGER.debug("Change the logfile to logfilepath: {}", logFilePath);
                setLogFilePath(logFilePath);
            }
            if (properties.containsKey(PROPERTY_LOG_FILE)) {
                // remove the RXTX path
                properties.remove(PROPERTY_RXTX_LOG_FILE);
                properties.remove(PROPERTY_LOG_FILE);
            }
            changes = true;
        }

        // save if necessary
        if (changes) {
            properties.put(PROPERTY_VERSION, DEFAULT_VERSION);
            save(null);
        }
    }

    private void loadDefaultValues() {
        getLogFilePath();
    }

    public void addPreferencesListener(PreferencesListener l) {
        listeners.add(l);
    }

    /**
     * Create a new PreferencesModel based on the current values of Preferences.
     * @return the new PreferencesModel instance
     */
    public PreferencesModel createPreferencesModel() {
        PreferencesModel preferencesModel = new PreferencesModel();
        preferencesModel.setMock(isMock());
        preferencesModel.setCommPort(getCommPort());
        preferencesModel.setStartTime(getStartTime());
        preferencesModel.setTimeFactor(getTimeFactor());
        preferencesModel.setLogFilePath(getLogFilePath());
        preferencesModel.setLabelPath(getLabelPath());
        LOGGER.debug("Created new references model: {}", preferencesModel);
        return preferencesModel;
    }

    private String[] getVersion() {
        String[] parts = null;
        try {
            // the version property was added with 1.4, if none is available use 1.3 as version 
            // to make sure the preferences are migrated
            preferencesVersion = properties.getProperty(PROPERTY_VERSION, "1.3");
            LOGGER.debug("Get the stored version: {}", preferencesVersion);
            parts = preferencesVersion.split("\\.");
        }
        catch (Exception ex) {
            // use a preferences version that causes the migration
            preferencesVersion = "1.3";
            parts = preferencesVersion.split("\\.");
        }
        return parts;
    }

    public String getCommPort() {
        String commPort = properties.getProperty(PROPERTY_COMM_PORT);
        LOGGER.debug("Get the stored comm port: {}", commPort);
        return commPort;
    }

    public String getLabelPath() {
        String result = properties.getProperty(PROPERTY_LABEL_PATH);

        if (result == null || result.isEmpty()) {
            result = new File(System.getProperty("user.home"), "." + BiDiBWizard.class.getSimpleName()).getPath();
        }
        return result;
    }

    public String getLogFilePath() {
        String logFilePath = properties.getProperty(PROPERTY_LOG_FILE_PATH);

        if (logFilePath == null || logFilePath.isEmpty()) {
            logFilePath = new File(System.getProperty("java.io.tmpdir")).getPath();

            // set in properties to be saved later
            properties.setProperty(PROPERTY_LOG_FILE_PATH, logFilePath);
        }
        return logFilePath;
    }

    public Date getStartTime() {
        try {
            return format.parse(properties.getProperty(PROPERTY_START_TIME, "06:00"));
        }
        catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    public int getTimeFactor() {
        return Integer.parseInt(properties.getProperty(PROPERTY_TIME_FACTOR, "1"));
    }

    public String getWorkingDirectory() {
        return properties.getProperty(PROPERTY_WORKING_DIRECTORY);
    }

    public boolean isMock() {
        return Boolean.valueOf(properties.getProperty(PROPERTY_MOCK, "false"));
    }

    /**
     * Save the preference values provided by the preferences model if available.
     * @param preferencesModel the preferences model to save
     */
    public void save(PreferencesModel preferencesModel) {

        if (preferencesModel != null) {
            // save the provided preferences model

            setTimeFactor(preferencesModel.getTimeFactor());
            setStartTime(preferencesModel.getStartTime());
            setLogFilePath(preferencesModel.getLogFilePath());
            setLabelPath(preferencesModel.getLabelPath());
            // set mock before comm port because the later triggers reload ports
            setMock(preferencesModel.isMock());
            // reconnect is triggered
            setCommPort(preferencesModel.getCommPort() != null ? preferencesModel.getCommPort() : "");
        }

        FileOutputStream fos = null;

        try {
            fos = new FileOutputStream(new File(preferencesPath));
            properties.store(fos, "user properties for " + BiDiBWizard.class.getSimpleName() + " configuration");
        }
        catch (IOException e) {
            LOGGER.warn("Store properties failed.", e);
        }
        finally {
            if (fos != null) {
                try {
                    fos.flush(); // flush the stream
                    fos.close();
                }
                catch (IOException e) {
                    LOGGER.warn("Flush or close file stream failed.", e);
                }
            }
        }
    }

    /**
     * Reload the logger configuration to make sure the changed settings for logfile location are applied.
     * Make sure the Preferences are saved before calling this method.
     * @param force force reload
     */
    public void reloadLoggerConfiguration(boolean force) {

        if (!force && !isReloadLogConfigurationPending()) {
            LOGGER.debug("Reload log configuration is skipped.");
            return;
        }
        // assume SLF4J is bound to logback in the current environment
        LoggerContext context = (LoggerContext) LoggerFactory.getILoggerFactory();

        try {
            JoranConfigurator configurator = new JoranConfigurator();
            configurator.setContext(context);
            // Call context.reset() to clear any previous configuration, e.g. default 
            // configuration. For multi-step configuration, omit calling context.reset().
            context.reset();

            InputStream logbackConfig = BiDiBWizard.class.getResourceAsStream("/logback.xml");
            if (logbackConfig == null) {
                logbackConfig = BiDiBWizard.class.getResourceAsStream("/config/logback-default.xml");
            }
            configurator.doConfigure(logbackConfig);
        }
        catch (JoranException je) {
            // StatusPrinter will handle this
        }
        StatusPrinter.printInCaseOfErrorsOrWarnings(context);
    }

    public void setCommPort(String commPort) {
        LOGGER.debug("Set the commPort: {}", commPort);
        properties.setProperty(PROPERTY_COMM_PORT, commPort);
        fireCommPortChanged(commPort);
    }

    public static void setPath(String path) {
        preferencesPath = path;
    }

    public void setLabelPath(String labelPath) {
        properties.setProperty(PROPERTY_LABEL_PATH, labelPath);
    }

    /**
     * Set the path to the location where the log files are written
     * @param logFilePath the path to the log files
     */
    public void setLogFilePath(String logFilePath) {
        String oldLogFilePath = properties.getProperty(PROPERTY_LOG_FILE_PATH);
        LOGGER.debug("Set the new logfile path: {}, oldLogFilePath: {}", logFilePath, oldLogFilePath);

        properties.setProperty(PROPERTY_LOG_FILE_PATH, logFilePath);

        if (logFilePath != null && !logFilePath.equalsIgnoreCase(oldLogFilePath)) {
            // the logfile has changed -> set the reload log configuration pending flag
            LOGGER.debug("The log configuration has changed.");
            reloadLogConfigurationPending = true;
        }
    }

    public void setMock(boolean mock) {
        Boolean oldValue = getBooleanProperty(PROPERTY_MOCK);
        properties.setProperty(PROPERTY_MOCK, Boolean.valueOf(mock).toString());
        if (oldValue == null || oldValue.booleanValue() != mock) {
            LOGGER.debug("Mock has changed.");
            fireMockChanged(mock);
        }
    }

    private Boolean getBooleanProperty(String key) {
        Object prop = properties.getProperty(key);
        Boolean value = null;
        if (prop != null) {
            value = Boolean.valueOf(prop.toString());
        }
        return value;
    }

    public void setStartTime(Date startTime) {
        properties.setProperty(PROPERTY_START_TIME, format.format(startTime));
        fireStartTimeChanged(startTime);
    }

    public void setTimeFactor(int timeFactor) {
        properties.setProperty(PROPERTY_TIME_FACTOR, String.valueOf(timeFactor));
        fireTimeFactorChanged(timeFactor);
    }

    public void setWorkingDirectory(String workingDirectory) {
        properties.setProperty(PROPERTY_WORKING_DIRECTORY, workingDirectory);
    }

    private void fireCommPortChanged(String commPort) {
        for (PreferencesListener l : listeners) {
            LOGGER.debug("Notify listener that the comm port has changed: {}", l);
            l.commPortChanged(commPort);
        }
    }

    private void fireMockChanged(boolean mock) {
        for (PreferencesListener l : listeners) {
            l.mockChanged(mock);
        }
    }

    private void fireStartTimeChanged(Date startTime) {
        for (PreferencesListener l : listeners) {
            l.startTimeChanged(startTime);
        }
    }

    private void fireTimeFactorChanged(int timeFactor) {
        for (PreferencesListener l : listeners) {
            l.timeFactorChanged(timeFactor);
        }
    }

    /**
     * @return the reloadLogConfigurationPending
     */
    public boolean isReloadLogConfigurationPending() {
        return reloadLogConfigurationPending;
    }

    /**
     * @param reloadLogConfigurationPending the reloadLogConfigurationPending to set
     */
    public void setReloadLogConfigurationPending(boolean reloadLogConfigurationPending) {
        this.reloadLogConfigurationPending = reloadLogConfigurationPending;
    }
}
